#include "config.h"

class_config::class_config()
{
	if (!wxFileExists("cosmos.cfg"))
		CreateDefault();
	else
		Load();
}

void class_config::CreateDefault()
{
	path_db = wxGetCwd();
	use_proxy = true;
	proxy_url = "www-proxy.chu-nancy.fr";
	proxy_port = 8080;
	
	rule default_rule = {"defaut", true, true, true, true, AND, AND, AND, true, true, 2, 5, true, "Nancy", false, "", 20};
	list_rules.push_back(default_rule);
			
	is_first_time = true;
}

class_config::~class_config()
{
	list_rules.clear();
}


void class_config::Save()
{
	wxXmlDocument* file_config = new wxXmlDocument();
	wxXmlNode* rootnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "CosMoS_config");		//root
		wxXmlNode* dbpathnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "dbpath");		// dbpath
			dbpathnode->AddAttribute("value", path_db);									//  dbpath_value
		rootnode->AddChild(dbpathnode);
		wxXmlNode* proxynode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "proxy");		// proxy
			proxynode->AddAttribute("set", bool2W(use_proxy));							//  proxy_set
			proxynode->AddAttribute("url", proxy_url);									//  proxy_url
			proxynode->AddAttribute("port", _itoW(proxy_port));							//  proxy_port
		rootnode->AddChild(proxynode);
		wxXmlNode* rulesetnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "ruleset");	// ruleset
			for (vector<rule>::iterator i = list_rules.begin(); i < list_rules.end(); i++)
				AddRulesToXML(rulesetnode, *i);											//  rule(s)
		rootnode->AddChild(rulesetnode);
	file_config->SetRoot(rootnode);
	
	file_config->Save("cosmos.cfg");
	delete file_config;
}

void class_config::AddRulesToXML(wxXmlNode* setlistnode, rule to_add)
{
	wxXmlNode* setnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "rule");				//  rule
				setnode->AddAttribute("name", to_add.name);								//   rule_name
				setnode->AddAttribute("inuse", bool2W(to_add.in_use));					//   rule_inuse
				wxXmlNode* nomsnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "noms");	//   noms
					nomsnode->AddAttribute("demandeur", bool2W(to_add.use_demandeur));	//    noms_demandeur
					nomsnode->AddAttribute("responsable", bool2W(to_add.use_responsable));		//    noms_responsable
					nomsnode->AddAttribute("consultant", bool2W(to_add.use_consultant));	//    noms_consultant
				setnode->AddChild(nomsnode);
				wxXmlNode* opsnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "ops");	//   ops
					opsnode->AddAttribute("demandeur", op2W(to_add.op_demandeur));		//    ops_demandeur
					opsnode->AddAttribute("both", op2W(to_add.op_both));					//    ops_both
					opsnode->AddAttribute("responsable", op2W(to_add.op_responsable));	//    ops_responsable
				setnode->AddChild(opsnode);
				wxXmlNode* datesnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "dates");//   dates
					datesnode->AddAttribute("usedebut", bool2W(to_add.use_debut));		//    dates_usedebut
					datesnode->AddAttribute("debut", _itoW(to_add.debut));				//    dates_debut
					datesnode->AddAttribute("usefin", bool2W(to_add.use_fin));			//    dates_usefin
					datesnode->AddAttribute("fin", _itoW(to_add.fin));					//    dates_fin
				setnode->AddChild(datesnode);
				wxXmlNode* villenode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "ville");//   ville
					villenode->AddAttribute("set", bool2W(to_add.use_ville));				//    ville_set
					villenode->AddAttribute("value", to_add.ville);						//	  ville_value
				setnode->AddChild(villenode);
				wxXmlNode* supplnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "suppl");//   suppl
					supplnode->AddAttribute("set", bool2W(to_add.use_suppl));				//    suppl_set
					supplnode->AddAttribute("value", to_add.suppl);						//	  suppl_value
				setnode->AddChild(supplnode);
				wxXmlNode* retmaxnode = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, "retmax");// retmax
					retmaxnode->AddAttribute("value", _itoW(to_add.retmax));				//    retmax_value
				setnode->AddChild(retmaxnode);
			setlistnode->AddChild(setnode);
}

void class_config::Load()
{
	rule current_rule;
	wxXmlDocument* file_config = new wxXmlDocument("cosmos.cfg");					//root
		wxXmlNode* parcours = file_config->GetRoot()->GetChildren(); 				// dbpath
			path_db = parcours->GetAttribute("value");									//  dbpath_value
		parcours = parcours->GetNext();													// proxy
			use_proxy = W2bool(parcours->GetAttribute("set"));							//  proxy_set
			proxy_url = parcours->GetAttribute("url");									//  proxy_url
			proxy_port = wxAtoi(parcours->GetAttribute("port"));							//  proxy_port
		parcours = parcours->GetNext();													// ruleset
		if (parcours->GetChildren())
		{
			parcours = parcours->GetChildren();											//  rule
			do																			
			{
				current_rule.name = parcours->GetAttribute("name");						//   rule_name
				current_rule.in_use = W2bool(parcours->GetAttribute("inuse"));			//   rule_inuse
				parcours = parcours->GetChildren();										//   noms
				current_rule.use_demandeur = W2bool(parcours->GetAttribute("demandeur"));	//    noms_demandeur
				current_rule.use_responsable = W2bool(parcours->GetAttribute("responsable"));// noms_responsable
				current_rule.use_consultant = W2bool(parcours->GetAttribute("consultant"));//	  noms_consultant
				parcours = parcours->GetNext();											//   ops
				current_rule.op_demandeur = W2op(parcours->GetAttribute("demandeur"));	//    ops_demandeur
				current_rule.op_both = W2op(parcours->GetAttribute("both"));				//    ops_both
				current_rule.op_responsable = W2op(parcours->GetAttribute("responsable"));//	  ops_responsable
				parcours = parcours->GetNext();											//   dates
				current_rule.use_debut = W2bool(parcours->GetAttribute("usedebut"));		//    dates_usedebut
				current_rule.debut = wxAtoi(parcours->GetAttribute("debut"));			//    dates_debut
				current_rule.use_fin = W2bool(parcours->GetAttribute("usefin"));			//    dates_usefin
				current_rule.fin = wxAtoi(parcours->GetAttribute("fin"));				//    dates_fin
				parcours = parcours->GetNext();											//   ville
				current_rule.use_ville = W2bool(parcours->GetAttribute("set"));			//    ville_set
				current_rule.ville = parcours->GetAttribute("value");					//    ville_value
				parcours = parcours->GetNext();											//   suppl
				current_rule.use_suppl = W2bool(parcours->GetAttribute("set"));			//    suppl_set
				current_rule.suppl = parcours->GetAttribute("value");					//    suppl_value
				parcours = parcours->GetNext();											//   retmax
				current_rule.retmax = wxAtoi(parcours->GetAttribute("value"));			//     retmax_value
				parcours = parcours->GetParent()->GetNext();
				list_rules.push_back(current_rule);
			}while (parcours);
		}
			
	delete file_config;
	
	is_first_time = false;
}

void class_config::AddRule(rule to_add)
{
	list_rules.push_back(to_add);
}

void class_config::DelRule(int index)
{
	list_rules.erase(list_rules.begin()+index);
}

int class_config::GetNbRules()
{
	return list_rules.size();
}

rule class_config::GetRule(int index)
{
	return list_rules[index];
}

void class_config::SetRule(int index, rule temp)
{
	list_rules[index] = temp;
}

void class_config::MoveRuleUp(int index)
{
	if (index > 0)
	{
		rule temp = list_rules[index-1];
		list_rules[index-1] = list_rules[index];
		list_rules[index] = temp;
	}
}

void class_config::MoveRuleDown(int index)
{
	if (index < (int)list_rules.size() - 1)
	{
		rule temp = list_rules[index+1];
		list_rules[index+1] = list_rules[index];
		list_rules[index] = temp;
	}
}