#include "optionsRche.h"

optionsRche::optionsRche( wxWindow* parent, class_config* config )
:
dialog_optionsRche( parent )
{
	this->config = config;
	for (int i=0; i < config->GetNbRules(); i++)
	{
		checklist_ruleset->Append(config->GetRule(i).name);
		if (config->GetRule(i).in_use)
			checklist_ruleset->Check(i);
	}
	checklist_ruleset->SetSelection(0);
	wxCommandEvent ev;
	ev.SetInt(0);
	OnChecklistSelect(ev);
}

void optionsRche::OnChecklistSelect( wxCommandEvent& event )
{
	rule current_rule = config->GetRule(event.GetInt());
	
	textctrl_nomset->SetValue(current_rule.name);
	
	checkbox_demandeur->SetValue(current_rule.use_demandeur);
	checkbox_responsable->SetValue(current_rule.use_responsable);
	checkbox_consultant->SetValue(current_rule.use_consultant);
	
	radio_demandeur->SetSelection(current_rule.op_demandeur);
	radio_both->SetSelection(current_rule.op_both);
	radio_responsable->SetSelection(current_rule.op_responsable);
	
	checkbox_debut->SetValue(current_rule.use_debut);
	checkbox_fin->SetValue(current_rule.use_fin);
	
	spinctrl_debut->SetValue(current_rule.debut);
	spinctrl_fin->SetValue(current_rule.fin);
		
	checkbox_ville->SetValue(current_rule.use_ville);
	textctrl_ville->SetValue(current_rule.ville);
	
	checkbox_suppl->SetValue(current_rule.use_suppl);
	textctrl_suppl->SetValue(current_rule.suppl);
	
	spinctrl_retmax->SetValue(current_rule.retmax);
	
	wxCommandEvent ev;
	UpdateUI(ev);
}

void optionsRche::OnAdd( wxCommandEvent& event )
{
	rule to_add;
	to_add.name = textctrl_nomset->GetValue();
	to_add.use_demandeur = checkbox_demandeur->GetValue();
	to_add.use_responsable = checkbox_responsable->GetValue();
	to_add.use_consultant = checkbox_consultant->GetValue();
	to_add.op_demandeur = radio_demandeur->GetSelection();
	to_add.op_both = radio_both->GetSelection();
	to_add.op_responsable = radio_responsable->GetSelection();
	to_add.use_debut = checkbox_debut->GetValue();
	to_add.use_fin = checkbox_fin->GetValue();
	to_add.debut = spinctrl_debut->GetValue();
	to_add.fin = spinctrl_fin->GetValue();
	to_add.use_ville = checkbox_ville->GetValue();
	to_add.ville = textctrl_ville->GetValue();
	to_add.use_suppl = checkbox_suppl->GetValue();
	to_add.suppl = textctrl_suppl->GetValue();
	to_add.retmax = spinctrl_retmax->GetValue();
	
	checklist_ruleset->Append(to_add.name);
	checklist_ruleset->SetSelection(checklist_ruleset->GetCount() - 1);
	config->AddRule(to_add);
}

void optionsRche::OnUp( wxCommandEvent& event )
{
	int selected = checklist_ruleset->GetSelection();
	if (selected != wxNOT_FOUND)
	{
		config->MoveRuleUp(selected);
		if (selected > 0)
		{
			wxString temp = checklist_ruleset->GetString(selected-1);
			checklist_ruleset->SetString(selected-1, checklist_ruleset->GetStringSelection());
			checklist_ruleset->SetString(selected, temp);
			checklist_ruleset->SetSelection(selected-1);
		}
	}
}

void optionsRche::OnDown( wxCommandEvent& event )
{
	int selected = checklist_ruleset->GetSelection();
	if (selected != wxNOT_FOUND)
	{
		config->MoveRuleDown(selected);
		if (selected < (int)checklist_ruleset->GetCount() - 1)
		{
			wxString temp = checklist_ruleset->GetString(selected+1);
			checklist_ruleset->SetString(selected+1, checklist_ruleset->GetStringSelection());
			checklist_ruleset->SetString(selected, temp);
			checklist_ruleset->SetSelection(selected+1);
		}
	}
}

void optionsRche::OnDel( wxCommandEvent& event )
{
	int selected = checklist_ruleset->GetSelection();
	if (selected != wxNOT_FOUND)
	{
		config->DelRule(selected);
		checklist_ruleset->Delete(selected);
	}
}

void optionsRche::OnUpdate( wxCommandEvent& event )
{
	rule to_mod;
	to_mod.name = textctrl_nomset->GetValue();
	to_mod.use_demandeur = checkbox_demandeur->GetValue();
	to_mod.use_responsable = checkbox_responsable->GetValue();
	to_mod.use_consultant = checkbox_consultant->GetValue();
	to_mod.op_demandeur = radio_demandeur->GetSelection();
	to_mod.op_both = radio_both->GetSelection();
	to_mod.op_responsable = radio_responsable->GetSelection();
	to_mod.use_debut = checkbox_debut->GetValue();
	to_mod.use_fin = checkbox_fin->GetValue();
	to_mod.debut = spinctrl_debut->GetValue();
	to_mod.fin = spinctrl_fin->GetValue();
	to_mod.use_ville = checkbox_ville->GetValue();
	to_mod.ville = textctrl_ville->GetValue();
	to_mod.use_suppl = checkbox_suppl->GetValue();
	to_mod.suppl = textctrl_suppl->GetValue();
	to_mod.retmax = spinctrl_retmax->GetValue();
	
	if (checklist_ruleset->GetSelection() != wxNOT_FOUND)
	{
		checklist_ruleset->SetString(checklist_ruleset->GetSelection(), to_mod.name);
		config->SetRule(checklist_ruleset->GetSelection(), to_mod);
	}
}

void optionsRche::OnOk( wxCommandEvent& event )
{
	Close();
}

void optionsRche::OnClose( wxCloseEvent& event )
{
	// Actualisation des sets selectionnés
	for (int i=0; i < (int)checklist_ruleset->GetCount(); i++)
	{
		rule temp = config->GetRule(i);
		temp.in_use = (checklist_ruleset->IsChecked(i));
		config->SetRule(i, temp);
	}
	config->Save();
	Show(false);
}

void optionsRche::UpdateUI( wxCommandEvent& event )
{
	if (checkbox_demandeur->IsChecked())
		radio_demandeur->Enable();
	else
		radio_demandeur->Disable();
	
	if (checkbox_responsable->IsChecked())
		radio_responsable->Enable();
	else
		radio_responsable->Disable();
	
	if (checkbox_demandeur->IsChecked() && checkbox_responsable->IsChecked())
		radio_both->Enable();
	else
		radio_both->Disable();
	
	if (checkbox_debut->IsChecked())
		spinctrl_debut->Enable();
	else
		spinctrl_debut->Disable();
		
	if (checkbox_fin->IsChecked())
		spinctrl_fin->Enable();
	else
		spinctrl_fin->Disable();

	if (checkbox_suppl->IsChecked())
		textctrl_suppl->Enable();
	else
		textctrl_suppl->Disable();
		
	if (checkbox_ville->IsChecked())
		textctrl_ville->Enable();
	else
		textctrl_ville->Disable();
}