#include "recherche.h"

#pragma GCC diagnostic ignored "-Wunused-function"

class_recherche::class_recherche(class_config* config, class_bdd* bdd, wxGrid* grid_consults)
{
	this->config = config;
	this->bdd = bdd;
	this->grid_consults = grid_consults;
	curl_handle = curl_easy_init ();
	curl_easy_setopt (curl_handle, CURLOPT_WRITEFUNCTION, WriteDataCallback);
	curl_easy_setopt (curl_handle, CURLOPT_WRITEDATA, (void*)&curl_data);
}

class_recherche::~class_recherche()
{
	curl_easy_cleanup (curl_handle);
	curl_global_cleanup ();
}

void class_recherche::Start()
{
	bool is_first_pass = true;
	wxArrayInt selection = grid_consults->GetSelectedRows();
	if (selection.GetCount() == 0)
		for (int i=0; i < grid_consults->GetNumberRows(); i++)
			selection.Add(i);
	
	for (int j=0; j < config->GetNbRules(); j++)
	{
		if (config->GetRule(j).in_use)
		{
			wxProgressDialog dialog_progress("Recherche", _T("Recherche avec la règle «") + config->GetRule(j).name + _T("»"), selection.GetCount(), NULL, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_SMOOTH | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME | wxPD_CAN_ABORT);
			dialog_progress.Update(0);
			for (unsigned int i=0; i < selection.GetCount(); i++)
			{
				if (is_first_pass || (grid_consults->GetCellValue(selection[i], 0) == "0"))
					Go (selection[i], config->GetRule(j));
				
				if (!dialog_progress.Update(i))
					break;
			}
		}
		is_first_pass = false;
	}
}

void class_recherche::Go(int row, rule regle)
{
	wxString id_consult = grid_consults->GetCellValue(row, 13);
	wxString requete = Build(row, regle);
	eSearch(requete);
	ParseSearch();

	wxArrayString list_id_result_bdd = bdd->GetIdResult(id_consult, true);
	
	wxArrayString temp;
	
	if (list_id_results.GetCount() > 0)
	{
		for (int i=(int)list_id_results.GetCount()-1; i >= 0; i--)
		{
			if (list_id_result_bdd.Index(list_id_results[i]) == wxNOT_FOUND)
			{
				temp.Add(list_id_results[i]);
				bdd->AddIdResult(id_consult, list_id_results[i]);
			}
		}
		list_id_results = temp;
		bdd->SetNbResults(id_consult, bdd->GetNbResults(id_consult) + list_id_results.GetCount());
		grid_consults->SetCellValue(row, 0, _itoW(bdd->GetNbResults(id_consult)));
		
		temp.Clear();
		
		for (unsigned int i=0; i < list_id_results.GetCount(); i++)
			if (!(bdd->IsPubli(list_id_results[i])))
				temp.Add(list_id_results[i]);
				
		list_id_results = temp;
		
		if (list_id_results.GetCount() > 0)
		{
			eFetch();
			ParseFetch();
		}
	}
}

void class_recherche::ParseFetch()
{
	XMLDocument* doc = new XMLDocument;
	doc->Parse(curl_data.content);
	
	XMLElement* parcours = doc->RootElement();
	parcours = parcours->FirstChildElement("PubmedArticle");
	do
	{
		wxString id, title, authors, abstract, journal, link, date, affiliation;
		
		parcours = parcours->FirstChildElement("MedlineCitation");										//MedlineCitation
			id << parcours->FirstChildElement("PMID")->GetText();										// PMID
			link << "http://www.ncbi.nlm.nih.gov/pubmed/" << id;
			parcours = parcours->FirstChildElement("Article"); 											// Article
				parcours = parcours->FirstChildElement("Journal");										//  Journal
					parcours = parcours->FirstChildElement("JournalIssue");								//   JournalIssue
						parcours = parcours->FirstChildElement("PubDate");								//    PubDate
							if (parcours->FirstChildElement("Day"))
								date << parcours->FirstChildElement("Day")->GetText();					//     *Day
							if (parcours->FirstChildElement("Month"))
								date << parcours->FirstChildElement("Month")->GetText();				//     *Month
							if (parcours->FirstChildElement("Year"))
								date << parcours->FirstChildElement("Year")->GetText();					//	   *Year
						parcours = parcours->Parent()->ToElement();										//   JournalIssue
					if (parcours->NextSiblingElement("ISOAbbreviation"))
						journal << parcours->NextSiblingElement("ISOAbbreviation")->GetText();			//   *ISOAbbreviation
					else if (parcours->NextSiblingElement("Title"))
						journal << parcours->NextSiblingElement("Title")->GetText();
					parcours = parcours->Parent()->ToElement();											//  Journal
				title << parcours->NextSiblingElement("ArticleTitle")->GetText();						//  *ArticleTitle
				if (parcours->NextSiblingElement("Abstract"))
				{
					parcours = parcours->NextSiblingElement("Abstract");								//  Abstract
						XMLElement* parcoursabstract = parcours->FirstChildElement("AbstractText");		//   AbstractText
						do
						{
							if (parcoursabstract->FirstAttribute())
								abstract << parcoursabstract->FirstAttribute()->Value() << " :\n";
							abstract << parcoursabstract->GetText() << "\n";
							parcoursabstract=parcoursabstract->NextSiblingElement("AbstractText");
						}while(parcoursabstract);
				}
				else
					abstract = "";
				if (parcours->NextSiblingElement("Affiliation"))
					affiliation << parcours->NextSiblingElement("Affiliation")->GetText();				//  *Affiliation
				else
					affiliation = "";
				parcours = parcours->NextSiblingElement("AuthorList");									//  AuthorList
					XMLElement* parcoursauteurs = parcours->FirstChildElement("Author");				//   Author
					do
					{
						if (parcoursauteurs->FirstChildElement("LastName"))
							authors << parcoursauteurs->FirstChildElement("LastName")->GetText();
						if (parcoursauteurs->FirstChildElement("ForeName"))
							authors << " " << parcoursauteurs->FirstChildElement("ForeName")->GetText() << ",";
						else if (parcoursauteurs->FirstChildElement("Initials"))
							authors << " " << parcoursauteurs->FirstChildElement("Initials")->GetText() << ",";
						else
							authors << ",";
						parcoursauteurs=parcoursauteurs->NextSiblingElement("Author");
					}while(parcoursauteurs);
					authors = authors.BeforeLast(',');
		parcours = parcours->Parent()->Parent()->Parent()->ToElement()->NextSiblingElement("PubmedArticle");
		
		bdd->AddPubli(id, title, authors, abstract, journal, link, date, affiliation);
	}while(parcours);
	
	delete(doc);
	if (curl_data.content)
		free (curl_data.content);
}

void class_recherche::eFetch()
{
	bool access_denied;
	do
	{
		access_denied = false;
		wxString requete = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&retmode=xml&id=";
		for (unsigned int i=0; i < list_id_results.GetCount(); i++)
			requete << list_id_results[i] << ",";
		requete = requete.BeforeLast(',');
		
		curl_easy_setopt (curl_handle, CURLOPT_URL, _C(requete));
				
		if (config->IsUseProxy()) 
		{
			curl_easy_setopt (curl_handle, CURLOPT_PROXY, _C(config->GetProxyUrl()));
			curl_easy_setopt (curl_handle, CURLOPT_PROXYPORT, config->GetProxyPort());
		}
		else
			curl_easy_setopt (curl_handle, CURLOPT_PROXY, "");
		
		curl_data.content = (char*) malloc(1);
		curl_data.size = 0;
		
		if (curl_handle)
			curl_easy_perform(curl_handle);
		
		if (wxString(curl_data.content).Left(6) == "<HTML>")
		{
			access_denied = true;
			time_t timer = time(NULL);
			wxProgressDialog dialog_progress("Erreur proxy", _T("En attente du proxy"), 200, NULL, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_SMOOTH | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME);
			while (time(NULL) < (timer + 200))
				dialog_progress.Update(time(NULL) - timer);
			dialog_progress.Update(200);
			if (curl_data.content)
				free(curl_data.content);
		}
	}while (access_denied);
}

void class_recherche::ParseSearch()
{
	list_id_results.Clear();
	
	XMLDocument* doc = new XMLDocument();
	doc->Parse(curl_data.content);
	
	XMLElement* parcours = doc->RootElement();
	if (parcours->FirstChildElement("ERROR"))
	{
		nb_results = 0;
		delete(doc);
		if (curl_data.content)
			free (curl_data.content);
		return;
	}
	parcours = parcours->FirstChildElement("Count");
		nb_results = atoi(parcours->GetText());
	parcours = parcours->NextSiblingElement("RetMax");
		int count = atoi(parcours->GetText());
	parcours = parcours->NextSiblingElement("IdList");
	if (count>0)
	{
		parcours = parcours->FirstChildElement();
		for (int i=0; i < count - 1 ; i++)
		{
			list_id_results.Add(parcours->GetText());
			parcours = parcours->NextSiblingElement();
		}
		list_id_results.Add(parcours->GetText());
		parcours = parcours->Parent()->ToElement();
	}
	delete(doc);
	if (curl_data.content)
		free (curl_data.content);
}

void class_recherche::eSearch(wxString requete)
{
	bool access_denied;
	do
	{
		access_denied = false;
		curl_easy_setopt (curl_handle, CURLOPT_URL, _C(requete));
				
		if (config->IsUseProxy()) 
		{
			curl_easy_setopt (curl_handle, CURLOPT_PROXY, _C(config->GetProxyUrl()));
			curl_easy_setopt (curl_handle, CURLOPT_PROXYPORT, config->GetProxyPort());
		}
		else
			curl_easy_setopt (curl_handle, CURLOPT_PROXY, "");
		
		curl_data.content = (char*) malloc(1);
		curl_data.size = 0;
		
		if (curl_handle)
			curl_easy_perform(curl_handle);
		
		if (wxString(curl_data.content).Left(6) == "<HTML>")
		{
			access_denied = true;
			time_t timer = time(NULL);
			wxProgressDialog dialog_progress("Erreur proxy", _T("En attente du proxy"), 200, NULL, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_SMOOTH | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME);
			while (time(NULL) < (timer + 200))
				dialog_progress.Update(time(NULL) - timer);
			dialog_progress.Update(200);
			if (curl_data.content)
				free (curl_data.content);
		}
	}while (access_denied);
}

wxString class_recherche::Build(int row, rule regle)
{
	wxString requete = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term=";
	bool both = false;
	if (grid_consults->GetCellValue(row, 5) == "")
		regle.use_demandeur = false;
	
	if (grid_consults->GetCellValue(row, 7) == "")
		regle.use_responsable = false;
	
	if (grid_consults->GetCellValue(row, 8) == "")
		regle.use_consultant = false;
	
	if (regle.use_demandeur && regle.use_responsable)
	{
		both = true;
		requete << "(";
	}
	
	if (regle.use_demandeur)
		requete << ParseDemandeur(grid_consults->GetCellValue(row, 5), grid_consults->GetCellValue(row, 6), regle.op_demandeur);
	
	if (both)
		requete << " " << op2W(regle.op_both) << " ";
	
	if (regle.use_responsable)
		requete << ParseResponsable(grid_consults->GetCellValue(row,7), regle.op_responsable);
		
	if (both)
		requete << ")";
	
	if (regle.use_consultant)
		requete << " AND " << ParseConsultant(grid_consults->GetCellValue(row,8));
	
	if (regle.use_ville)
		requete << " AND " << regle.ville << "[Affiliation]";
		
	if (regle.use_suppl)
		requete << " AND " << regle.suppl;
	
	requete << "&retmax=" << _itoW(regle.retmax);
	
	if ((regle.use_debut) || (regle.use_fin))
	{
		wxDateTime temp;
		temp.ParseFormat(grid_consults->GetCellValue(row, 12), "%d/%m/%Y");
				
		requete << "&datetype=pdat";
		if (regle.use_debut)
			temp += wxDateSpan(regle.debut);
		requete << "&mindate=" << temp.Format("%Y/%m/%d");
		
		temp.ParseFormat(grid_consults->GetCellValue(row, 12), "%d/%m/%Y");
		if (regle.use_fin)
			temp += wxDateSpan(regle.fin);
		else
			temp.SetToCurrent();
		requete << "&maxdate=" << temp.Format("%Y/%m/%d");
	}
	
	requete.Replace(" ", "+");
	return requete;
}

wxString class_recherche::ParseDemandeur(wxString nom, wxString prenom, int op)
{
	wxString out, nom1, nom2, prenom1, prenom2;
	bool multi = false;
	if (nom.Find('/') != wxNOT_FOUND)
	{
		multi = true;
		nom1 = nom.BeforeFirst('/');
		nom2 = nom.AfterFirst('/');
	}
	
	if (prenom.Find('/') == wxNOT_FOUND)
		prenom1 = prenom;
	else
	{
		prenom1 = prenom.BeforeFirst('/');
		prenom2 = prenom.AfterFirst('/');
	}
	
	if (prenom1.Find('-') != wxNOT_FOUND)
		prenom1 = prenom1.BeforeFirst('-').Left(1) + prenom1.AfterFirst('-').Left(1);
	else
		prenom1 = prenom1.Left(1);
	
	if (prenom2.Find('-') != wxNOT_FOUND)
		prenom2 = prenom2.BeforeFirst('-').Left(1) + prenom2.AfterFirst('-').Left(1);
	else
		prenom2 = prenom2.Left(1);
	
	if (multi)
	{
		out << "(" << nom1 << " " << prenom1 << "[Author]";
		out << " " << op2W(op) << " ";
		out << nom2 << " " << prenom2 << "[Author])";
	}
	else
		out << nom << " " << prenom1 << "[Author]";
	
	return out;
}

wxString class_recherche::ParseResponsable(wxString responsable, int op)
{
	wxString out;
	if (responsable.Find('/') == wxNOT_FOUND)
		out = responsable + "[Author]";
	else
	{
		out = "(" + responsable.BeforeFirst('/') + "[Author]";
		out << " " << op2W(op) << " ";
		out << responsable.AfterFirst('/') << "[Author])";
	}
	
	return out;
}

wxString class_recherche::ParseConsultant(wxString consultant)
{
	wxString out, nom, prenom;
	prenom = consultant.BeforeFirst(' ');
	nom = consultant.AfterFirst(' ');
	
	if (prenom.Find('-') != wxNOT_FOUND)
		prenom = prenom.BeforeFirst('-').Left(1) + prenom.AfterFirst('-').Left(1);
	else
		prenom = prenom.Left(1);
	
	out = nom + " " + prenom + "[Author]";
	
	return out;
}

static size_t WriteDataCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
  size_t realsize = size * nmemb;
  CurlData *mem = (CurlData *)userp;
 
  mem->content = (char*) realloc(mem->content, mem->size + realsize + 1);
  if (mem->content == NULL) {
    printf("Echec de realloc (I'd like to see that happen !!!!!)");
    exit(EXIT_FAILURE);
  }
 
  memcpy(&(mem->content[mem->size]), contents, realsize);
  mem->size += realsize;
  mem->content[mem->size] = 0;
 
  return realsize;
}